package com.streambase.contrib.wiimote;


import wiiusej.WiiUseApiManager;
import wiiusej.Wiimote;
import wiiusej.wiiusejevents.physicalevents.ExpansionEvent;
import wiiusej.wiiusejevents.physicalevents.IREvent;
import wiiusej.wiiusejevents.physicalevents.MotionSensingEvent;
import wiiusej.wiiusejevents.physicalevents.WiimoteButtonsEvent;
import wiiusej.wiiusejevents.utils.WiimoteListener;
import wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.ClassicControllerRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.DisconnectionEvent;
import wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.GuitarHeroRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.NunchukInsertedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.NunchukRemovedEvent;
import wiiusej.wiiusejevents.wiiuseapievents.StatusEvent;

import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.adapter.InputAdapter;
import com.streambase.sb.operator.Parameterizable;
import com.streambase.sb.operator.TypecheckException;

/**
 * Generated by JDT StreamBase Client Templates (Version: 6.4.10.1001130606).
 *
 * Input adapters take information from outside of an application and
 * provide it in the form of a stream or streams to the application. 
 * Any implementation of this class <b>must</b> extend class InputAdapter.
 * Input adapters should call sendOutput(int, Tuple) to provide tuples 
 * to the application.
 * <p>
 * For in-depth information on implementing a custom adapter, please
 * see "Developing StreamBase Embedded Adapters" in the StreamBase documentation.
 * <p>
 */
public class sbWiiAdapter extends InputAdapter implements Parameterizable,
		Runnable {

	public static final long serialVersionUID = 1265232550322L;
	private Schema schemaIREvent;
	private Wiimote wiimote;
	private Schema schemaButtonsEvent;
	private Schema schemaMotionSensing;
	private boolean useSmoothing = false;
	private WiimoteListener wiimotelistener;

	public Schema getSchemaMotionSensing() {
		return schemaMotionSensing;
	}

	public void setSchemaMotionSensing(Schema schemaMotionSensing) {
		this.schemaMotionSensing = schemaMotionSensing;
	}

	public Schema getSchemaButtonsEvent() {
		return schemaButtonsEvent;
	}

	public void setSchemaButtonsEvent(Schema schemaButtonsEvent) {
		this.schemaButtonsEvent = schemaButtonsEvent;
	}

	/**
	 * The constructor is called when the Adapter instance is created, but before the Adapter 
	 * is connected to the StreamBase application. We recommended that you set the initial input 
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount). 
	 * The default is no input ports or output ports. The constructor may also set default values for 
	 * adapter parameters. These values will be displayed in StreamBase Studio when a new adapter is 
	 * dragged to the canvas, and serve as the default values for omitted optional parameters.
	 */
	public sbWiiAdapter() {
		super();
		setPortHints(0, 3);
		setDisplayName(this.getClass().getSimpleName());
		setShortDisplayName(this.getClass().getSimpleName());

	}

	/**
	 * Typecheck this adapter. The typecheck method is called after the adapter instance is
	 * connected in the StreamBase application. The adapter should validate its parameters 
	 * and throw TypecheckException if any problems are found. The message associated with 
	 * the thrown TypecheckException will be displayed in StreamBase Studio during authoring,
	 * or printed on the console by sbd. Input adapters should set the schema of each output
	 * port by calling the setOutputSchema(portNum, schema) method for each output port. 
	 * If the adapter needs to change the number of input ports based on parameter values,
	 * it should call requireInputPortCount(portCount) at this point. 
	 */
	public void typecheck() throws TypecheckException {
		// TODO Make sure all properties have valid values.

		// Sets the output schema for the given output port (port #'s are zero based).
		setOutputSchema(0, schemaIREvent);
		setOutputSchema(1, schemaButtonsEvent);
		setOutputSchema(2, schemaMotionSensing);
	}

	/**
	 * Initialize the adapter. If typecheck succeeds, the init method is called before
	 * the StreamBase application is started. Note that your adapter is not required to
	 * define the init method, unless you need to register a runnable or perform
	 * initialization of a resource such as, for example, a JDBC pool.
	 */
	public void init() throws StreamBaseException {
		super.init();

		// Register the object so it will be run as a thread managed by StreamBase.
		registerRunnable(this, true);
		
	    Wiimote[] wiimotes = WiiUseApiManager.getWiimotes(1, true);
	    if (wiimotes.length == 0) {
	    	throw new StreamBaseException("No Wiimotes available");
	    }
	    wiimote = wiimotes[0];
	    wiimote.activateIRTRacking();
	    wiimote.activateMotionSensing();
	    if (useSmoothing) wiimote.activateSmoothing();
	    
	}
	
	/**
	 * Shutdown adapter by cleaning up any resources used (e.g. close an output file
	 * if it has been opened). When the application is shutting down, the adapter's shutdown.
	 * method will be called first. Once this has returned, all threads should exit and the
	 * adapter is considered shutdown.
	 */
	public void shutdown() {
		if (wiimote == null) return;
		wiimote.removeWiiMoteEventListeners(wiimotelistener);
	    wiimote.disconnect();
	    WiiUseApiManager.shutdown();
	}

	/**
	 * Main thread of the adapter. At this point, the application begins to run.
	 * StreamBase will start threads for any managed runnables registered by
	 * earlier calls to registerRunnable. Input adapters can call sendOutput(port, tuple) 
	 * at any time to output a tuple to the specified port.
	 */
	public void run() {
	    wiimote.addWiiMoteEventListeners(wiimotelistener = new WiimoteListener(){
	    	@Override
	    	public void onButtonsEvent(WiimoteButtonsEvent arg0) {	    	    
	    		createButtonsTuple(arg0);
	    	}

			@Override
			public void onClassicControllerInsertedEvent(
					ClassicControllerInsertedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onClassicControllerRemovedEvent(
					ClassicControllerRemovedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onDisconnectionEvent(DisconnectionEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onExpansionEvent(ExpansionEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onGuitarHeroInsertedEvent(GuitarHeroInsertedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onGuitarHeroRemovedEvent(GuitarHeroRemovedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onIrEvent(IREvent arg0) {
				createIREventTuple(arg0);
			}

			@Override
			public void onMotionSensingEvent(MotionSensingEvent arg0) {
				createMotionSensingTuple(arg0);
				
			}

			@Override
			public void onNunchukInsertedEvent(NunchukInsertedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onNunchukRemovedEvent(NunchukRemovedEvent arg0) {
				// TODO Auto-generated method stub
				
			}

			@Override
			public void onStatusEvent(StatusEvent arg0) {
				// TODO Auto-generated method stub
				
			}
	    });
	}


	private void createMotionSensingTuple(MotionSensingEvent motionSensingEvent) {
		Tuple out = schemaMotionSensing.createTuple();
		Schema.Field[] fields = out.getSchema().getFields(); // individual fields in the schema

		for (int i = 0; i < fields.length; ++i) {
			Schema.Field f = fields[i];
			if ("gForceX".equals(f.getName())) {
					try {
						out.setDouble(f, motionSensingEvent.getGforce().getX());
					} catch (TupleException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			if ("gForceY".equals(f.getName())) {
				try {
					out.setDouble(f, motionSensingEvent.getGforce().getY());
				} catch (TupleException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if ("gForceZ".equals(f.getName())) {
				try {
					out.setDouble(f, motionSensingEvent.getGforce().getZ());
				} catch (TupleException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		try {
			sendOutput(2, out);
		} catch (StreamBaseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}

	protected void createButtonsTuple(WiimoteButtonsEvent buttonsEvent) {
		Tuple out = schemaButtonsEvent.createTuple();
		Schema.Field[] fields = out.getSchema().getFields(); // individual fields in the schema

		for (int i = 0; i < fields.length; ++i) {
			Schema.Field f = fields[i];
			if ("buttonsJustPressed".equals(f.getName())) {
					try {
						out.setInt(f, buttonsEvent.getButtonsJustPressed());
					} catch (TupleException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			if ("buttonsJustReleased".equals(f.getName())) {
				try {
					out.setInt(f, buttonsEvent.getButtonsJustReleased());
				} catch (TupleException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			if ("buttonsHeld".equals(f.getName())) {
				try {
					out.setInt(f, buttonsEvent.getButtonsHeld());
				} catch (TupleException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		try {
			sendOutput(1, out);
		} catch (StreamBaseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}

	private Object createIREventTuple(IREvent irEvent) {
		Tuple out = schemaIREvent.createTuple();
		Schema.Field[] fields = out.getSchema().getFields(); // individual fields in the schema

		for (int i = 0; i < fields.length; ++i) {
			Schema.Field f = fields[i];
			if ("irX".equals(f.getName())) {
					try {
						out.setInt(f, irEvent.getX());
					} catch (TupleException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			if ("irY".equals(f.getName())) {
				try {
					out.setInt(f, irEvent.getY());
				} catch (TupleException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		
		try {
			sendOutput(0, out);
		} catch (StreamBaseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public Schema getSchemaIREvent() {
		return this.schemaIREvent;
	}

	public void setSchemaIREvent(Schema chemaIREvent) {
		this.schemaIREvent = chemaIREvent;
	}

	public boolean isUseSmoothing() {
		return useSmoothing;
	}

	public void setUseSmoothing(boolean useSmoothing) {
		this.useSmoothing = useSmoothing;
	}
	
	
}
